SystemOrganization addCategory: #'Neo-CSV-Core'!

Object subclass: #NeoCSVReader
	instanceVariableNames: 'readStream charBuffer separator stringStream fieldCount recordClass recordClassIsIndexable fieldAccessors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-CSV-Core'!

!NeoCSVReader commentStamp: '<historical>' prior: 0!
I am NeoCSVReader.

I read a format that
- is text based (ASCII, Latin1, Unicode)
- consists of records, 1 per line (any line ending convention)
- where records consist of fields separated by a delimiter (comma, tab, semicolon)
- where every record has the same number of fields
- where fields can be quoted should they contain separators or line endings

Without further configuration, records will become Arrays of Strings.

By specifiying a recordClass and fields with optional converters most objects can be read and instanciated correctly.

MIT License.
!

!NeoCSVReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/25/2012 21:15'!
on: readStream
	"Initialize on readStream, which should be a character stream that 
	implements #next, #atEnd and (optionally) #close."

	^ self new
		on: readStream;
		yourself! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 22:35'!
addField
	"Add the next indexable field with a pass through converter"
	
	self
		recordClassIsIndexable: true; 
		addFieldAccessor: [ :string | string ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 22:35'!
addField: accessor
	"Add a field based on a mutator accessor accepting a field 
	String as argument to be sent to an instance of recordClass"
	
	self
		recordClassIsIndexable: false; 
		addFieldAccessor: [ :object :string |
			object perform: accessor with: string ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 22:35'!
addField: accessor converter: converter
	"Add a field based on a mutator accessor accepting the result of 
	applying the converter block on the field String read as argument 
	to be sent to an instance of recordClass"

	self
		recordClassIsIndexable: false; 
		addFieldAccessor: [ :object :string |
			object perform: accessor with: (converter value: string) ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/16/2012 13:33'!
addFieldAccessor: block
	fieldAccessors 
		ifNil: [
			fieldAccessors := Array with: block ]
		ifNotNil: [
			fieldAccessors := fieldAccessors copyWith: block ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/26/2012 20:49'!
addFieldAt: key
	"Add a field that will be stored under key in recordClass as String"
	
	self
		recordClassIsIndexable: false; 
		addFieldAccessor: [ :object :string |
			object at: key put: string ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/26/2012 20:50'!
addFieldAt: key converter: converter
	"Add a field that will be stored under key in recordClass as the result of 
	applying the converter block on the field String read as argument"

	self
		recordClassIsIndexable: false; 
		addFieldAccessor: [ :object :string |
			object at: key put: (converter value: string) ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 23:00'!
addFieldConverter: converter
	"Add the next indexable field with converter block, 
	accepting a String and returning a specific object"

	self
		recordClassIsIndexable: true; 
		addFieldAccessor: converter! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/16/2012 13:51'!
addFields: accessors
	accessors do: [ :each |
		self addField: each ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 20:53'!
addFieldsAt: keys
	keys do: [ :each |
		self addFieldAt: each ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/21/2012 22:22'!
addFloatField
	self addFieldConverter: [ :string | Float readFrom: string ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/16/2012 18:53'!
addFloatField: accessor
	self 
		addField: accessor 
		converter: [ :string | Float readFrom: string ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 21:04'!
addFloatFieldAt: key
	self 
		addFieldAt: key 
		converter: [ :string | Float readFrom: string ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/21/2012 22:38'!
addIntegerField
	self addFieldConverter: [ :string | Integer readFrom: string ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/16/2012 18:53'!
addIntegerField: accessor
	self 
		addField: accessor 
		converter: [ :string | Integer readFrom: string ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 21:04'!
addIntegerFieldAt: key
	self 
		addFieldAt: key 
		converter: [ :string | Integer readFrom: string ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/25/2012 14:42'!
addSymbolField
	self addFieldConverter: [ :string | string asSymbol ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/25/2012 14:42'!
addSymbolField: accessor
	self 
		addField: accessor 
		converter: [ :string | string asSymbol ]! !

!NeoCSVReader methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 21:04'!
addSymbolFieldAt: key
	self 
		addFieldAt: key 
		converter: [ :string | string asSymbol ]! !

!NeoCSVReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/25/2012 20:47'!
atEnd
	^ charBuffer == nil and: [ readStream atEnd ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 12:32'!
close
	readStream ifNotNil: [
		readStream close.
		readStream := nil ]! !

!NeoCSVReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/25/2012 14:45'!
do: block
	"Execute block for each record until I am at end."
	
	[ self atEnd ]
		whileFalse: [ 
			block value: self next ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 23:01'!
fieldCount: anObject
	"Set the field count up front.
	This will be used when reading records as Arrays.
	This instance variable will be set and used automatically based on the first record seen.
	If set, the fieldAccessors collection defines (overrides) the fieldCount."

	fieldCount := anObject! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 22:29'!
initialize
	super initialize.
	recordClass := Array.
	recordClassIsIndexable := true.
	separator := $,! !

!NeoCSVReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/21/2012 22:30'!
next
	"Read the next record.
	I will return an instance of recordClass."
	
	^ recordClassIsIndexable
		ifTrue: [ self readNextRecordAsArray ] 
		ifFalse: [ self readNextRecordAsObject ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:28'!
nextChar
	^ charBuffer 
		ifNil: [ 
			readStream next ]
		ifNotNil: [ | char |
			char := charBuffer.
			charBuffer := nil.
			^ char ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/25/2012 21:15'!
on: aReadStream
	"Initialize on aReadStream, which should be a character stream that 
	implements #next, #atEnd and (optionally) #close."
	
	readStream := aReadStream! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:23'!
peekChar
	^ charBuffer 
		ifNil: [ 
			charBuffer := readStream next ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:56'!
peekEndOfLine
	| char |
	char := self peekChar codePoint.
	^ (char == 10 "Character lf" ) or: [ char == 13 "Character cr" ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:56'!
peekQuote
	^ self peekChar == $"! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:56'!
readEndOfLine
	| char |
	char := self peekChar codePoint.
	char == 10 "Character lf"
		ifTrue: [ 
			self nextChar. 
			^ true ].
	char == 13 "Character cr"
		ifTrue: [
			self nextChar.
			(self atEnd not and: [ self peekChar codePoint == 10 "Character lf" ])
				ifTrue: [ 
					self nextChar ]. 
			^ true  ].
	^ false
! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 17:21'!
readEndOfQuotedField
	(self readQuote and: [ self peekQuote not ])
		ifTrue: [
			self readSeparator.
			^ true ].
	^ false! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 17:24'!
readField
	^ self peekQuote
		ifTrue: [
			self readQuotedField ]
		ifFalse: [
			self readUnquotedField ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/21/2012 22:30'!
readFirstRecord 
	^ recordClass streamContents: [ :stream |
		[ self atEnd or: [ self readEndOfLine ] ]
			whileFalse: [ 
				stream nextPut: self readField ] ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/21/2012 22:49'!
readNextRecord
	| record |
	record := recordClass new: fieldCount.
	1 to: fieldCount do: [ :each | | rawValue convertedValue |
		rawValue := self readField.
		convertedValue := fieldAccessors 
			ifNil: [ rawValue ]
			ifNotNil: [ (fieldAccessors at: each) value: rawValue ].
		record at: each put: convertedValue ].
	self assert: (self atEnd or: [ self readEndOfLine ]).
	^ record! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/21/2012 22:46'!
readNextRecordAsArray
	fieldAccessors ifNotNil: [ fieldCount := fieldAccessors size ].
	^ fieldCount 
		ifNil: [ | record |
			record := self readFirstRecord.
			fieldCount := record size.
			record ] 
		ifNotNil: [
			self readNextRecord ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/16/2012 13:45'!
readNextRecordAsObject
	| object |
	object := recordClass new.
	fieldAccessors do: [ :each |
		each value: object value: self readField ].
	self assert: (self atEnd or: [ self readEndOfLine ]).
	^ object ! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:12'!
readQuote
	self peekQuote
		ifTrue: [ 
			self nextChar. 
			^ true ].
	^ false! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:13'!
readQuotedField
	self readQuote.
	^ self stringStreamContents: [ :stream |
		[ self atEnd or: [ self readEndOfQuotedField ] ]
			whileFalse: [
				stream nextPut: self nextChar ] ]! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:56'!
readSeparator
	self peekChar == separator
		ifTrue: [ 
			self nextChar. 
			^ true ].
	^ false! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:13'!
readUnquotedField
	^ self stringStreamContents: [ :stream |
		[ self atEnd or: [ self readSeparator or: [ self peekEndOfLine ] ] ]
			whileFalse: [ 
				stream nextPut: self nextChar ] ]! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:15'!
recordClass: anObject
	"Set the object class to instanciate while reading records.
	When doing so, you have to specify fields as well."
	
	recordClass := anObject! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/21/2012 23:00'!
recordClassIsIndexable: boolean
	"Set whether recordClass should be treated as an indexable sequenceable collection
	that #new: and #streamContents and whose instances implement #at:put: 
	If false, fields accessors have to be provided.
	Will be set automatically when field accessors or converters are set."
	
	recordClassIsIndexable := boolean ! !

!NeoCSVReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:24'!
separator: character
	"Set the field separator character to use, defaults to comma"
	
	self assert: character isCharacter.
	separator := character ! !

!NeoCSVReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 12:32'!
stringStreamContents: block
	"Like String streamContents: block
	but reusing the underlying buffer for improved efficiency"
	
	stringStream 
		ifNil: [ 
			stringStream := (String new: 32) writeStream ].
	stringStream reset.
	block value: stringStream.
	^ stringStream contents! !

!NeoCSVReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/25/2012 14:45'!
upToEnd 
	"Read and collect records into an Array until there are none left.
	Return the array."
	
	^ Array streamContents: [ :stream |
		self do: [ :each | stream nextPut: each ] ]! !

Object subclass: #NeoCSVWriter
	instanceVariableNames: 'writeStream separator fieldWriter lineEnd fieldAccessors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-CSV-Core'!

!NeoCSVWriter commentStamp: '<historical>' prior: 0!
I am NeoCSVWriter.

I write a format that
- is text based (ASCII, Latin1, Unicode)
- consists of records, 1 per line (any line ending convention)
- where records consist of fields separated by a delimiter (comma, tab, semicolon)
- where every record has the same number of fields
- where fields can be quoted should they contain separators or line endings

Without further configuration, I write record objects whose fields can be enumerated using #do: such as SequenceableCollections

By specifiying fields any object can be written converting and/or quoting each field as needed.

MIT License.!

!NeoCSVWriter class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/14/2012 09:47'!
on: writeStream
	"Initialize on writeStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	^ self new
		on: writeStream;
		yourself! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:32'!
addField: accessor
	"Add a field based on an accessor to be written using fieldWriter"

	self addFieldAccessor: [ :object |
		self writeField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 23:03'!
addFieldAccessor: block
	fieldAccessors 
		ifNil: [
			fieldAccessors := Array with: block ]
		ifNotNil: [
			fieldAccessors := fieldAccessors copyWith: block ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/26/2012 20:30'!
addFieldAt: key
	"Add a field based on a key to be written using fieldWriter"

	self addFieldAccessor: [ :object |
		self writeField: (object at: key) ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/16/2012 13:34'!
addFields: accessors
	accessors do: [ :each |
		self addField: each ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 20:31'!
addFieldsAt: keys
	keys do: [ :each |
		self addFieldAt: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:33'!
addObjectField: accessor
	"Add a field based on an accessor to be written as an #object field"

	self addFieldAccessor: [ :object |
		self writeObjectField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/26/2012 20:30'!
addObjectFieldAt: key
	"Add a field based on a key to be written as an #object field"

	self addFieldAccessor: [ :object |
		self writeObjectField: (object at: key) ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/15/2012 22:38'!
addObjectFields: accessors
	accessors do: [ :each |
		self addObjectField: each ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 20:31'!
addObjectFieldsAt: keys
	keys do: [ :each |
		self addObjectFieldAt: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:33'!
addQuotedField: accessor
	"Add a field based on an accessor to be written as a #quoted field"

	self addFieldAccessor: [ :object |
		self writeQuotedField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/26/2012 20:30'!
addQuotedFieldAt: key
	"Add a field based on a key to be written as a #quoted field"

	self addFieldAccessor: [ :object |
		self writeQuotedField: (object at: key) ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/15/2012 22:39'!
addQuotedFields: accessors
	accessors do: [ :each |
		self addQuotedField: each ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 20:32'!
addQuotedFieldsAt: keys
	keys do: [ :each |
		self addQuotedFieldAt: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:33'!
addRawField: accessor
	"Add a field based on an accessor to be written as a #raw field"

	self addFieldAccessor: [ :object |
		self writeRawField: (object perform: accessor) ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/26/2012 20:31'!
addRawFieldAt: key
	"Add a field based on a key to be written as a #raw field"

	self addFieldAccessor: [ :object |
		self writeRawField: (object at: key) ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/15/2012 22:38'!
addRawFields: accessors
	accessors do: [ :each |
		self addRawField: each ]! !

!NeoCSVWriter methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/26/2012 20:32'!
addRawFieldsAt: keys
	keys do: [ :each |
		self addRawFieldAt: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 10:10'!
close
	writeStream ifNotNil: [
		writeStream close.
		writeStream := nil ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:31'!
fieldWriter: symbol
	"Set the field write to be used, either #quoted (the default), #raw or #object.
	This determines how field values will be written in the general case.
	#quoted will wrap fields #asString in double quotes and escape embedded double quotes
	#raw will write fields #asString as such (no separator, double quote or end of line chars allowed)
	#object will #print: fields (no separator, double quote or end of line chars allowed)"
	
	self assert: (#(quoted raw object) includes: symbol).
	fieldWriter := ('write', symbol capitalized, 'Field:') asSymbol
	! !

!NeoCSVWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2012 19:39'!
flush
	writeStream flush! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/15/2012 19:16'!
initialize 
	super initialize.
	lineEnd := String crlf.
	separator := $, .
	fieldWriter := #writeQuotedField: 
! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:25'!
lineEndConvention: symbol
	"Set the end of line convention to be used.
	Either #cr, #lf or #crlf (the default)."
	
	self assert: (#(cr lf crlf) includes: symbol).
	lineEnd := String perform: symbol! !

!NeoCSVWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2012 16:23'!
nextPut: anObject
	"Write anObject as single record.
	Depending on configuration fieldAccessors or a #do: enumeration will be used."
	
	fieldAccessors 
		ifNil: [
			self writeFieldsUsingDo: anObject ]
		ifNotNil: [
			self writeFieldsUsingAccessors: anObject ].
	self writeEndOfLine ! !

!NeoCSVWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/17/2012 16:23'!
nextPutAll: collection
	"Write a collection of objects as records"
	
	collection do: [ :each | 
		self nextPut: each ]! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/14/2012 09:48'!
on: aWriteStream
	"Initialize on aWriteStream, which should be a character stream that 
	implements #nextPut:, #nextPutAll:, #space and (optionally) #close."

	writeStream := aWriteStream
! !

!NeoCSVWriter methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/17/2012 16:24'!
separator: character
	"Set character to be used as separator"
	
	self assert: character isCharacter.
	separator := character ! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 19:16'!
writeEndOfLine
	writeStream nextPutAll: lineEnd ! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 12:47'!
writeField: object
	self perform: fieldWriter with: object! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 22:37'!
writeFieldsUsingAccessors: anObject
	| first |
	first := true.
	fieldAccessors do: [ :each | | fieldValue | 
		first ifTrue: [ first := false ] ifFalse: [ self writeSeparator ].
		fieldValue := each value: anObject ]! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 19:45'!
writeFieldsUsingDo: anObject
	| first |
	first := true.
	anObject do: [ :each |
		first ifTrue: [ first := false ] ifFalse: [ self writeSeparator ].
		self writeField: each ]! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 10:51'!
writeObjectField: object
	object printOn: writeStream ! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/25/2012 20:59'!
writeQuotedField: object
	| string |
	string := object asString.
	writeStream nextPut: $".
	string do: [ :each |
		each == $" 
			ifTrue: [ writeStream nextPut: $"; nextPut: $" ]
			ifFalse: [ writeStream nextPut: each ] ].
	writeStream nextPut: $"! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/15/2012 12:39'!
writeRawField: object
	writeStream nextPutAll: object asString! !

!NeoCSVWriter methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/14/2012 10:03'!
writeSeparator
	writeStream nextPut: separator ! !
